/*
  Stepper Motor Test
  stepper-test01.ino
  Uses MA860H or similar Stepper Driver Unit
  Has speed control & reverse switch

  DroneBot Workshop 2019
  https://dronebotworkshop.com
*/
#include <PWM.h>
#include <LiquidCrystal_74HC595.h>
#include <SimpleModbusSlave.h>
#include <SM.h>
#include <Filter.h>

AnalogFilter<200, 5> filter;// the sample time is 2ms (scnd part) and the number of samples is 10 (first part).

SM Gr7Secu(&etape10);
SM Gr7Prod(&etape20);//pwm launched to the HSS86
SM DisplayRPM(&etape100);//display position and speed
SM DisplaySTEPS(&etape200);
SM AnalogReadFILTER(&etape400);
SM RunStop(&etape40);//RUN STOP DCY FCY

#define DS 11
#define SHCP 13
#define STCP 12
#define RS 1
#define E 3 //2
#define D4 4 //3
#define D5 5 //4
#define D6 6 // 5
#define D7 7 //6

LiquidCrystal_74HC595 lcd(DS, SHCP, STCP, RS, E, D4, D5, D6, D7);

// Define pins
//int ENABLERS485 = 10;  // 10 enable RS485 but without any declaration
int EAplus = 2;  // EA+ output CHA coder
int driverPUL = 9;    // PUL- pin pwm
int driverDIR = 6;    // DIR- pin WHEN LOW CCW SENS; LOW CW SENS
int driverPEND = 5;    // PEND- pin LIGHT ON on RUN, LIGHT OFF WHEN POSITION REACHED
int driverALRM = 4;    // ALRM- pin IF ON , NO EMERGENCY ALARME
int driverATU = 7;    // ATU- pin IF HIGH, NOT ON EMERGENCY STOP
int driverPOM = 8;    // POM- pin
int LIGHT = 17;    // A3 LIGHT- pin
int RUNSTOP = 18;    // A4 RUN STOP- pin  A0 RUN 1 STOP
int SPlight = 15;    // A1 SET POINT LIGHT-pin A1 POWEROFF: ON, RUN MODE: ON, SET POINT: OFF
int ALRMlight = 16;    // A2 ALARM LIGHT-pin A2 ALARM: ON, NO ALARM OFF
int STEPCNTR = 3;//STEPcount from pwm
int spd = A0;     // Potentiometer
int count = 0;
signed long Microstep;


//dtostrf (f, 10, 4, buf);  // number, width, decimal places, buffer

float master_count; //count EA+
float master_countdisp;
signed long positionDEG = 0;
signed long DEGsp = 0;
signed long DEGTURNmes = 0;
signed long step_count = 0; //count STEPS SENT
signed long TURNcount = 0;
signed long TURNsp = 0;
signed long DEGTURNsp = 0;
float RPM;
signed long SPEEDVALUE;
float K;
unsigned long DELTAdegTURN;


// Variables
int value;
int filtered;
unsigned long pd;       // Pulse Delay period
boolean setdir = LOW; // Set Direction
int etat = 0;//SMlib just to use rising edge function
int etat2 = 1;
int memo;
int memo2 = 0;
int memo3 = 0;
int memo4 = 0;
int memo5 = 0;
unsigned long currentTime = 0;
unsigned long previousTime = 0;
unsigned long currentTime2 = 0;
unsigned long previousTime2 = 0;

enum
{
  // just add or remove registers and your good to go...
  // The first register starts at address 0
  NOuse,//0
  CAM,  //1 AUTOMATIC MANUAL
  CCW,  //2 CCW or CW
  SPEEDPOT, //3
  CCWlight,//4 light on CCW
  CWlight,//5 light on CW
  SWITCHvalue,//6 type switch value: Microsteps/rev
  DEGvalue,//7
  TURNvalue,//8
  SPEEDvalue,//9
  BPPOM,//10 launch POM
  POMlight,//11 light on POM launched
  DCY,//12 RUN
  FCY,//13 STOP
  LIGHTrun,//14
  LIGHTstop,//15
  CSpeedPos,// 16 define speed or position regulation
  SpeedpotSP,//17  speed set point on speed regulation
  NBRturnSP,//18 set point for turn on position regulation
  DEGSP,//19 set point for degrees remain on position regulation
  EMERGENCYlight,//20
  SWITCHsense,//21 define CCW or CW
  ACKNOLEDGE,//22 acknoledge in MANUAL MODE
  LIGHTspeed,//23
  LIGHTpos,//24
  FREQUENCY,//25
  HOLDING_REGS_SIZE // leave this one
  // total number of registers for function 3 and 16 share the same register array
  // i.e. the same address space
};

unsigned int holdingRegs[HOLDING_REGS_SIZE]; // function 3 and 16 register array

void setup() {
  InitTimersSafe();

  lcd.begin(20, 4);              // initialize the lcd
  lcd.setCursor ( 0, 0 );        // go to the next line
  lcd.print ("HYBRID STP MOTOR");
  pinMode (EAplus, INPUT);
  pinMode (driverPUL, OUTPUT);
  pinMode (driverDIR, OUTPUT);
  pinMode (driverPEND, INPUT);
  pinMode (driverALRM, INPUT);
  pinMode (driverATU, INPUT);
  pinMode (driverPOM, INPUT);
  pinMode (LIGHT, OUTPUT);
  pinMode (STEPCNTR, INPUT);
  // Set pin as INPUT pin FOR ANALOG FILTERING
  pinMode(spd, INPUT);
  pinMode (RUNSTOP, OUTPUT);
  digitalWrite(RUNSTOP,HIGH);//0: RUN , 1: STOP on ENA+
  pinMode (SPlight, OUTPUT);
  pinMode (ALRMlight, OUTPUT);
  /* parameters(HardwareSerial* SerialPort,
              long baudrate,
    unsigned char byteFormat,
              unsigned char ID, 1
              unsigned char transmit enable pin, 10
              unsigned int holding registers size,
              unsigned int* holding register array)
  */
  /* Valid modbus byte formats are:
     SERIAL_8N2: 1 start bit, 8 data bits, 2 stop bits
     SERIAL_8E1: 1 start bit, 8 data bits, 1 Even parity bit, 1 stop bit
     SERIAL_8O1: 1 start bit, 8 data bits, 1 Odd parity bit, 1 stop bit
     You can obviously use SERIAL_8N1 but this does not adhere to the
     Modbus specifications. That said, I have tested the SERIAL_8N1 option
     on various commercial masters and slaves that were suppose to adhere
     to this specification and was always able to communicate... Go figure.
     These byte formats are already defined in the Arduino global name space.
  */
  modbus_configure(&Serial, 115200, SERIAL_8N2, 1, 10, HOLDING_REGS_SIZE, holdingRegs);

  // modbus_update_comms(baud, byteFormat, id) is not needed but allows for easy update
  //of the port variables and slave id dynamically in any function.
  modbus_update_comms(115200, SERIAL_8N2, 1);

  holdingRegs[SWITCHvalue] = 400;

  attachInterrupt(0, flag, FALLING);// PULSE count EA+
  attachInterrupt(1, STEPcount, FALLING);//STEPS count on PWM

}

void loop() {
  EXEC(DisplayRPM);//display RPM calculation with PULSE on EA+/s (1000/REV)
  EXEC(AnalogReadFILTER);// only for POT on A0, map the pot and PWM settings
  EXEC(DisplaySTEPS);// display STEP from PWM sent to HSS86, rev and position
  EXEC(Gr7Secu);//mise en marche de la state machine: un pion est déposé dans

  // State10 et les séquences peuvent se dérouler
  if (digitalRead(driverATU) == 0 ) {
    pwmWrite(driverPUL, 0);
    Gr7Prod.Finish();
    RunStop.Finish();
  }
  if ((digitalRead(driverATU) == 1) && Gr7Prod.Finished && RunStop.Finished) {
    EXEC(Gr7Prod);
    Gr7Prod.Set(etape20);
    EXEC(RunStop);
    RunStop.Set(etape40);
  }

  modbus_update();

  //digitalWrite(LIGHT, digitalRead(driverPEND));//light on once position reached
  digitalWrite(SPlight, digitalRead(driverPEND));
  digitalWrite(ALRMlight, !digitalRead(driverALRM));
  
  Microstep = holdingRegs[SWITCHvalue];
  holdingRegs[DEGvalue] = positionDEG;
  holdingRegs[TURNvalue] = TURNcount;
  //holdingRegs[SPEEDvalue] = TURNcount;
  holdingRegs[FREQUENCY]=pd;//DISPLAYS PWM FREQUENCY IN HZ on hmi
  
  TURNsp = holdingRegs[NBRturnSP];
  DEGsp = holdingRegs[DEGSP];

  //calculation from Microsteps switch positions
  master_countdisp = 1.056 * master_count; //with EA+, pulse count
  RPM = 0.06336 * master_count;// RPS INTO RPM
  //RPM=(pd*60)/Microstep;//theoric                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                       

  holdingRegs[SpeedpotSP] = 10 * (pd * 60) / Microstep; //x10 send integer not float to hmi

  if ((holdingRegs[CCW] == 1) || setdir == 0) {
    holdingRegs[SpeedpotSP] = -(holdingRegs[SpeedpotSP]); //
  }

  TURNcount = step_count / Microstep;
  positionDEG = 360 * (step_count - (TURNcount * Microstep)) / Microstep;
  DEGTURNsp = DEGsp + (TURNsp * 360);
  DEGTURNmes = positionDEG + (TURNcount * 360);
  K = (DEGTURNmes - DEGTURNsp) / DELTAdegTURN;
}

State etape10() {
  pwmWrite(driverPUL, 0);
  TURNcount = 0;
  positionDEG = 0;
  step_count = 0;
  holdingRegs[EMERGENCYlight] = 1;
  holdingRegs[LIGHTrun] = 0;
  holdingRegs[LIGHTstop] = 1;
  if (digitalRead(driverATU) == HIGH)  Gr7Secu.Set(etape11);
}

State etape11() {
  holdingRegs[EMERGENCYlight] = 0;
  EXEC(Gr7Prod);
  EXEC(RunStop);
  if (digitalRead(driverATU) == LOW)  Gr7Secu.Set(etape10);
}

State etape40() {
  holdingRegs[LIGHTrun] = 0;
  holdingRegs[LIGHTstop] = 1;
  memo = 0;
  if ((holdingRegs[DCY] == 1) && (holdingRegs[CAM] == 1)) RunStop.Set(etape41);
}
State etape41() {
  holdingRegs[LIGHTrun] = 1;
  holdingRegs[LIGHTstop] = 0;
  memo = 1;
  if ((holdingRegs[FCY] == 1) || (holdingRegs[CAM] == 0) || (memo2 == 1) || (memo3 == 1) || (memo4 == 1) || (memo5 == 1))RunStop.Set(etape40);
}

State etape20() {
  RESETdisp();
  digitalWrite(RUNSTOP,HIGH);//stop
  pwmWrite(driverPUL, 0);
  if (holdingRegs[CAM] == 0) Gr7Prod.Set(etape21);
  if (holdingRegs[CAM] == 1) Gr7Prod.Set(etape202);
}

State etape202() {
  RESETdisp();
  memo2 = 0;
  memo3 = 0;
  memo5 = 0;
  pwmWrite(driverPUL, 0);
  if (holdingRegs[CAM] == 0) Gr7Prod.Set(etape20);
  if ((holdingRegs[CSpeedPos] == 1) && (holdingRegs[CAM] == 1))  Gr7Prod.Set(etape22);
  if ((holdingRegs[CSpeedPos] == 0) && (holdingRegs[CAM] == 1))  Gr7Prod.Set(etape27);
}

State etape21() {
  holdingRegs[LIGHTspeed] = 0;
  holdingRegs[LIGHTpos] = 0;
  digitalWrite(RUNSTOP,LOW);//RUN
  if (holdingRegs[CCW] == 1) {
    lcd.setCursor ( 17, 0 );        // go to the next line
    lcd.print ("CCW");
    holdingRegs[CCWlight] = 1;
    holdingRegs[CWlight] = 0;
    setdir = 0;
    digitalWrite(driverDIR, LOW);
  }
  else {
    lcd.setCursor ( 17, 0 );        // go to the next line
    lcd.print (" CW");
    holdingRegs[CCWlight] = 0;
    holdingRegs[CWlight] = 1;
    setdir = 1;
    digitalWrite(driverDIR, HIGH);
  }

  if (holdingRegs[ACKNOLEDGE] == 1) {
    pwmWrite(driverPUL, 127);
  }
  else {
    pwmWrite(driverPUL, 0);
  }

  if (holdingRegs[CAM] == 1) Gr7Prod.Set(etape20);
}

State etape22() {
  holdingRegs[LIGHTspeed] = 1;
  holdingRegs[LIGHTpos] = 0;
  digitalWrite(RUNSTOP,LOW);//RUN
  RESETdisp();
  pwmWrite(driverPUL, 0);
  if (holdingRegs[BPPOM] == 1) Gr7Prod.Set(etape23);
  if ((holdingRegs[CSpeedPos] == 0) || (holdingRegs[CAM] == 0)) {
    memo3 = 1;
    Gr7Prod.Set(etape202);
  }
}

State etape23() {
  setdir = 0;
  digitalWrite(driverDIR, LOW);
  pwmWrite(driverPUL, 127);
  if (digitalRead(driverPOM) == 1) Gr7Prod.Set(etape24);
  if ((holdingRegs[CSpeedPos] == 0) || (holdingRegs[CAM] == 0))  {
    memo3 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape24() {
  RESETdisp();
  holdingRegs[POMlight] = 1;
  pwmWrite(driverPUL, 0);
  if (memo == 1) Gr7Prod.Set(etape25);
  if (( holdingRegs[CSpeedPos] == 0) || (holdingRegs[CAM] == 0))  {
    memo3 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape25() {
  setdir = 1;
  digitalWrite(driverDIR, HIGH);
  holdingRegs[POMlight] = 0;
  pwmWrite(driverPUL, 127);
  if (memo == 0) Gr7Prod.Set(etape22);
  if (RE(holdingRegs[BPPOM], etat) == 1) Gr7Prod.Set(etape26);
  if ((holdingRegs[CSpeedPos] == 0) || (holdingRegs[CAM] == 0))  {
    memo3 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape26() {
  setdir = 0;
  digitalWrite(driverDIR, LOW);
  pwmWrite(driverPUL, 127);
  if (RE(holdingRegs[BPPOM], etat) == 1) Gr7Prod.Set(etape25);
  if (memo == 0) Gr7Prod.Set(etape22);
  if ((holdingRegs[CSpeedPos] == 0) || (holdingRegs[CAM] == 0))  {
    memo3 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape27() {
  holdingRegs[LIGHTspeed] = 0;
  holdingRegs[LIGHTpos] = 1;
  digitalWrite(RUNSTOP,LOW);//RUN
  RESETdisp();
  pwmWrite(driverPUL, 0);
  memo2 = 1;
  if (holdingRegs[BPPOM] == 1) Gr7Prod.Set(etape28);
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape28() {//CCW
  memo2 = 0;
  setdir = 0;
  digitalWrite(driverDIR, LOW);
  pwmWrite(driverPUL, 127);
  if (digitalRead(driverPOM) == 1) Gr7Prod.Set(etape29);
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape29() {//ON POM TYPE POSITION  >0 AND DCY TO GO ON
  holdingRegs[POMlight] = 1;
  pwmWrite(driverPUL, 0);
  RESETdisp();
  if (Gr7Prod.Timeout(50)) Gr7Prod.Set(etape292);
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}

State etape292() {//TYPE POSITION  >0 AND DCY TO GO ON
  //DEGTURNsp=DEGsp+(TURNsp*360)
  //DEGTURNmes=positionDEG+(TURNcount*360)
  memo4 = 0;
  if (RE(holdingRegs[DCY], etat) == 1) {
    DELTAdegTURN = DEGTURNmes - DEGTURNsp;
    Gr7Prod.Set(etape293);
  }
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape293() {//TYPE POSITION  >0 AND DCY TO GO ON
  //TURNcount = 0;
  //TURNsp = 0;
  //if (TURNcount >= TURNsp) Gr7Prod.Set(etape32);
  //if (TURNcount <= TURNsp) Gr7Prod.Set(etape30);
  if (DEGTURNmes >= DEGTURNsp) Gr7Prod.Set(etape32);
  if (DEGTURNmes <= DEGTURNsp) Gr7Prod.Set(etape30);
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0)) {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}

State etape30() {//CW
  holdingRegs[POMlight] = 0;
  setdir = 1;
  digitalWrite(driverDIR, HIGH);
  pwmWrite(driverPUL, 127);
  //if ((TURNcount >= TURNsp)) Gr7Prod.Set(etape31);
  if ((DEGTURNmes >= DEGTURNsp)) Gr7Prod.Set(etape31);
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}
State etape31() {//CW
  pwmWrite(driverPUL, 0);
  if (memo == 0) {
    Gr7Prod.Set(etape27);
    memo4 = 1;
  }
  if (memo == 1) {
    Gr7Prod.Set(etape292);
    memo4 = 1;
  }
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}

State etape32() {//CCW
  holdingRegs[POMlight] = 0;
  setdir = 0;
  digitalWrite(driverDIR, LOW);
  pwmWrite(driverPUL, 127);
  //if ((TURNcount <= TURNsp)) Gr7Prod.Set(etape31);
  if ((DEGTURNmes <= DEGTURNsp)) Gr7Prod.Set(etape31);
  if ((holdingRegs[CSpeedPos] == 1) || (holdingRegs[CAM] == 0))  {
    memo5 = 1;
    Gr7Prod.Set(etape202);
  }
}

State etape100() {//display EA+ count every 1s, rpm  on HMI
  currentTime = micros();
  if ((currentTime - previousTime) > 1000000) {
    previousTime = currentTime;
    //holdingRegs[FREQUENCY]=pd;//DISPLAYS PWM FREQUENCY IN HZ
    //holdingRegs[SPEEDvalue] = 10 * RPM; //
    holdingRegs[SPEEDvalue] = 10 * RPM;//10 * RPM; //
    CHAcountDisp();//displays RPM
    CHAcountDisp2();//displays PULSES
    master_count = 0;
    currentTime = micros();
  }
}

State etape200() {//display STEPS
  currentTime2 = micros();
  if ((currentTime2 - previousTime2) > 350000) {//less than 300000: STRANGE DISPLAY
    previousTime2 = currentTime2;
    STEPsentDisp();
    TURNdisp();
    POSITIONdisp();
    currentTime2 = micros();
  }
}

State etape400() {//analog in filter , set pwm
  // Read pin value A0 to filter
  value = analogRead(spd);
  // Filter it unless, stange values
  filtered = filter.update(value);
  //pd = map(filtered, 0, 1023, 5000, 25000);

  if ((holdingRegs[CSpeedPos] == 0) && (holdingRegs[CAM] == 1)) {
    pd = K * map(holdingRegs[SPEEDPOT], 0, 1023, 5000, 25000);
  }
  else {
    pd = map(holdingRegs[SPEEDPOT], 0, 1023, 5000, 25000);
  }
  SetPinFrequencySafe(driverPUL, pd);
}

void flag() {//pulse count
  // add 1 to count for CW
  if (setdir == LOW) {
    master_count-- ;
  }
  // subtract 1 from count for CCW
  if (setdir == HIGH) {
    master_count++ ;
  }
}

void STEPcount() {//step count
  // add 1 to count for CW
  if (setdir == LOW) {
    step_count--;
  }
  // subtract 1 from count for CCW
  if (setdir == HIGH) {
    step_count++;
  }
}

void CHAcountDisp() {//display RPM calculated on pulse count
  lcd.setCursor ( 0, 1 );        // go to the next line
  lcd.print ("RPM ON EA+:");
  if ((RPM < 10) && (RPM >= 0)) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print ("    ");
    lcd.setCursor ( 15, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if ((RPM < 0) && (RPM > -9)) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 14, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if ((RPM > 9) && (RPM < 100)) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 14, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if ((RPM < -9) && (RPM > -99)) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 13, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if ((RPM > 99) && (RPM < 1000)) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 13, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if ((RPM < -99) && (RPM > -999)) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 12, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if (RPM > 999) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 12, 1 );        // go to the next line
    lcd.print (RPM);
  }
  if (RPM < -999) {
    lcd.setCursor ( 11, 1 );        // go to the next line
    lcd.print (RPM);
  }
}

void CHAcountDisp2() {//display pulse count every second
  lcd.setCursor ( 0, 2 );        // go to the next line
  lcd.print ("PULSES EA+:");
  if ((master_countdisp < 10) && (master_countdisp >= 0)) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print ("    ");
    lcd.setCursor ( 15, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if ((master_countdisp < 0) && (master_countdisp > -9)) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 14, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if ((master_countdisp > 9) && (master_countdisp < 100)) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 14, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if ((master_countdisp < -9) && (master_countdisp > -99)) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 13, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if ((master_countdisp > 99) && (master_countdisp < 1000)) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 13, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if ((master_countdisp < -99) && (master_countdisp > -999)) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 12, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if (master_countdisp > 999) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 12, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
  if (master_countdisp < -999) {
    lcd.setCursor ( 11, 2 );        // go to the next line
    lcd.print (master_countdisp);
  }
}

void STEPsentDisp() {//display step counted
  lcd.setCursor ( 11, 3 );        // go to the next line
  lcd.print (":");
  if ((step_count < 10) && (step_count >= 0))  {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("       ");
    lcd.setCursor ( 19, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count < 0) && (step_count > -10)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("      ");
    lcd.setCursor ( 18, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count >= 10) && (step_count < 100)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("      ");
    lcd.setCursor ( 18, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -10) && (step_count > -100)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("     ");
    lcd.setCursor ( 17, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count >= 100) && (step_count < 1000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("     ");
    lcd.setCursor ( 17, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -100) && (step_count > -1000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("    ");
    lcd.setCursor ( 16, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count >= 1000) && (step_count < 10000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("    ");
    lcd.setCursor ( 16, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -1000) && (step_count > -10000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 15, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count >= 10000) && (step_count < 100000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 15, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -10000) && (step_count > -100000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 14, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count >= 100000) && (step_count < 1000000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 14, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -100000) && (step_count > -1000000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 13, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count >= 1000000) && (step_count < 10000000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 13, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -1000000) && (step_count > -10000000)) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if (step_count >= 10000000) {
    lcd.setCursor ( 12, 3 );        // go to the next line
    lcd.print (step_count);
  }
  if ((step_count <= -10000000) && (step_count > -100000000)) {
    lcd.setCursor ( 11, 3 );        // go to the next line
    lcd.print (step_count);
  }
}

void TURNdisp() {//display turns done
  lcd.setCursor ( 6, 3);        // go to the next line
  lcd.print (":");
  if ((TURNcount < 10) && (TURNcount >= 0)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("     ");
    lcd.setCursor ( 5, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount < 0) && (TURNcount > -9)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("    ");
    lcd.setCursor ( 4, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount > 9) && (TURNcount < 100)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("    ");
    lcd.setCursor ( 4, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount < -9) && (TURNcount > -99)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 3, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount > 99) && (TURNcount < 1000)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 3, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount < -99) && (TURNcount > -999)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 2, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount > 999) && (TURNcount < 10000)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 2, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount < -999) && (TURNcount > -9999)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 1, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount > 9999) && (TURNcount < 100000)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 1, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
  if ((TURNcount < -9999) && (TURNcount > -99999)) {
    lcd.setCursor ( 0, 3 );        // go to the next line
    lcd.print (TURNcount);
  }
}

void POSITIONdisp() {//display the position remain in degrees
  if ((positionDEG < 10) && (positionDEG >= 0)) {
    lcd.setCursor ( 7, 3 );        // go to the next line
    lcd.print ("   ");
    lcd.setCursor ( 10, 3 );        // go to the next line
    lcd.print (positionDEG);
  }
  if ((positionDEG < 0) && (positionDEG > -9)) {
    lcd.setCursor ( 7, 3 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 9, 3 );        // go to the next line
    lcd.print (positionDEG);
  }
  if ((positionDEG > 9) && (positionDEG < 100)) {
    lcd.setCursor ( 7, 3 );        // go to the next line
    lcd.print ("  ");
    lcd.setCursor ( 9, 3 );        // go to the next line
    lcd.print (positionDEG);
  }
  if ((positionDEG < -9) && (positionDEG > -99)) {
    lcd.setCursor ( 7, 3 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 8, 3 );        // go to the next line
    lcd.print (positionDEG);
  }
  if ((positionDEG > 99) && (positionDEG < 999)) {
    lcd.setCursor ( 7, 3 );        // go to the next line
    lcd.print (" ");
    lcd.setCursor ( 8, 3 );        // go to the next line
    lcd.print (positionDEG);
  }
  if ((positionDEG < -99) && (positionDEG > -999)) {
    lcd.setCursor ( 7, 3 );        // go to the next line
    lcd.print (positionDEG);
  }
}

void RESETdisp() {//display the position remain in degrees
  TURNcount = 0;
  positionDEG = 0;
  step_count = 0;
  master_count = 0;
  master_countdisp = 0;
  RPM = 0;
  lcd.setCursor ( 11, 1 );        // go to the next line
  lcd.print ("    ");
  lcd.setCursor ( 15, 1 );        // go to the next line
  lcd.print ("0.00 ");
  lcd.setCursor ( 11, 2 );        // go to the next line
  lcd.print ("    ");
  lcd.setCursor ( 15, 2 );        // go to the next line
  lcd.print ("0.00 ");
  lcd.setCursor ( 0, 3 );        // go to the next line
  lcd.print ("     ");
  lcd.setCursor ( 5, 3 );        // go to the next line
  lcd.print ("0");
  lcd.setCursor ( 6, 3 );        // go to the next line
  lcd.print (":");
  lcd.setCursor ( 7, 3 );        // go to the next line
  lcd.print ("   ");
  lcd.setCursor ( 10, 3 );        // go to the next line
  lcd.print ("0");
  lcd.setCursor ( 11, 3 );        // go to the next line
  lcd.print (":");
  lcd.setCursor ( 12, 3 );        // go to the next line
  lcd.print ("       ");
  lcd.setCursor ( 19, 3 );        // go to the next line
  lcd.print ("0");
}
